package Lingua::Pere::Config;

use locale;
use strict;

use vars qw(@ISA $VERSION @EXPORT @EXPORT_OK);

use Exporter;

$VERSION = "0.2.0";
@ISA = qw(Exporter);

@EXPORT = ();
@EXPORT_OK = qw(
	printhelp
	undefopt
	wrongopt
);

use POSIX;
use FindBin;
use File::Spec;
use Getopt::Long;
use Config::General;
use Pod::Usage;

#**************************************************************************
sub printhelp {
	my ($msg, $vrb) = @_;
	$vrb ||= 0;
	pod2usage( -message => $msg, -verbose => $vrb)
}

#**************************************************************************
sub undefopt {
	my ($opt) = @_;

	printhelp(sprintf("Undefined option: '%s'", $opt));
}

#**************************************************************************
sub wrongopt {
	my ($opt) = @_;

	printhelp(sprintf("Wrong option: '%s'", $opt));
}

#**************************************************************************
sub new {
	my ($class, $fil) = @_;
	$fil ||= "$0.config";

	my %opt = ();
	my $opa = Getopt::Long::Parser->new;
	$opa->configure(qw(auto_abbrev));
	pod2usage(-verbose => 0) unless ($opa->getoptions(
		\%opt,
		'pfx=s',
		'csv=s',
		'txt=s',
		'res=s',
		'iln=s',
		'oln=s',
		'thm=s',
		'enc=s',
		'mem=i',
		'ovr',
		'cas!',
		'lin!',
		'cfg=s',
		'rut=s',
		'help|?',
		'man',
	));
	pod2usage(-verbose => 0) if ($opt{help});
	pod2usage(-exitstatus => 0, -verbose => 2) if ($opt{man});

	my $pat;
	if (exists($opt{cfg}) && defined($opt{cfg}) && ($opt{cfg} ne '') && (-f $opt{cfg})) {
		$pat = $opt{cfg};
	} else {
		$pat = File::Spec->canonpath(sprintf("%s/%s", getcwd(), $fil));
		$pat = File::Spec->canonpath(sprintf("%s/%s", $ENV{HOME}, $fil)) if ((!-f $pat) && exists($ENV{HOME}) && ($ENV{HOME} ne ''));
		$pat = File::Spec->canonpath(sprintf("%s/%s", $ENV{USERPROFILE}, $fil)) if ((!-f $pat) && exists($ENV{USERPROFILE}) && ($ENV{USERPROFILE} ne ''));
		$pat = File::Spec->canonpath(sprintf("%s/%s", '/etc', $fil)) if ((!-f $pat) && (-d '/etc'));
		$pat = File::Spec->canonpath(sprintf("%s/%s", $FindBin::Bin, $fil)) unless (-f $pat);
	}

	my %cfg = ();
	if (-f $pat) {
		my $cnf = new Config::General(
			-ConfigFile => $pat,
			-AutoTrue   => 1,
		);
		%cfg = $cnf->getall;
	} else {
		if (exists($ENV{HOME}) && ($ENV{HOME} ne '')) {
			$pat = File::Spec->canonpath(sprintf("%s/%s", $ENV{HOME}, $fil));
		} elsif (exists($ENV{USERPROFILE}) && ($ENV{USERPROFILE} ne '')) {
			$pat = File::Spec->canonpath(sprintf("%s/%s", $ENV{USERPROFILE}, $fil));
		} else {
			$pat = sprintf("%s/%s", getcwd(), $fil);
		}
	}

	my %all = (
		%cfg,
		%opt
	);

	my $self = bless {
		'name' => $fil,
		'path' => $pat,
		'opt'  => \%all,
	}, $class;

	return $self;
}

#**************************************************************************
sub save {
	my ($self, $fil) = @_;
	$fil ||= $self->configpath;
	
	if (-f $fil) {
		my $bak = $fil.".bak";
		unlink($bak) if (-f $bak);
		rename($fil, $bak);
	}

	my $conf = Config::General->new(-AutoTrue   => 1);
	my %opt = %{$self->opt};
	delete($opt{pfx});
	delete($opt{res});
	delete($opt{txt});
	delete($opt{csv});
	$conf->save_file($fil, \%opt);
}

#**************************************************************************
sub opt {
	my ($self, $nam, $val) = @_;
	
	if (defined($nam) && ($nam ne '')) {
		if (defined($val)) {
			$self->{opt}->{$nam} = $val;
		} else {
			return (exists $self->{opt}->{$nam}) ?$self->{opt}->{$nam} : undef;
		}
	} else {
		return $self->{opt};
	}
}

#**************************************************************************
sub def {
	my ($self, $nam, $val) = @_;
	
	if (defined($nam) && ($nam ne '')) {
		my $res = $self->{opt}->{$nam};
		unless (defined $res) {
			$res = $val;
			$self->opt($nam, $res);
		}
		return $res;
	} else {
		return $self->{opt};
	}
}

#**************************************************************************
sub configname {
	my ($self) = @_;
	
	return $self->{name};
}

#**************************************************************************
sub configpath {
	my ($self) = @_;
	
	return $self->{path};
}

#**************************************************************************
sub csv {
	my ($self, $val) = @_;

	if (defined $val) {
		$self->opt('csv', $val);
	} else {
		my $res = $self->opt('csv');
		if (defined $res) {
			return $res;
		} else {
			undefopt('csv');
		}
	}
}

#**************************************************************************
sub txt {
	my ($self, $val) = @_;

	if (defined $val) {
		$self->opt('txt', $val);
	} else {
		my $res = $self->opt('txt');
		if (defined $res) {
			return $res;
		} else {
			undefopt('txt');
		}
	}
}

#**************************************************************************
sub res {
	my ($self, $val) = @_;

	if (defined $val) {
		$self->opt('res', $val);
	} else {
		my $res = $self->opt('res');
		if (defined $res) {
			return $res;
		} else {
			undefopt('res');
		}
	}
}

#**************************************************************************
sub iln {
	my ($self, $val) = @_;

	if (defined $val) {
		$self->opt('iln', $val);
	} else {
		my $res = $self->opt('iln');
		if (defined($res) && ($res ne '')) {
			return $res;
		} else {
			undefopt('iln');
		}
	}
}

#**************************************************************************
sub oln {
	my ($self, $val) = @_;

	if (defined $val) {
		$self->opt('oln', $val);
	} else {
		my $res = $self->opt('oln');
		if (defined($res) && ($res ne '')) {
			return $res;
		} else {
			undefopt('oln');
		}
	}
}

#**************************************************************************
sub thm {
	my ($self, $val) = @_;

	if (defined $val) {
		$self->opt('thm', $val);
	} else {
		my $res = $self->opt('thm');
		if (defined($res) && ($res ne '')) {
			return $res;
		} else {
			undefopt('thm');
		}
	}
}

#**************************************************************************
sub rut {
	my ($self, $val) = @_;

	if (defined $val) {
		$self->opt('rut', $val);
	} else {
		my $res = $self->opt('rut');
		if (defined $res) {
			return $res;
		} else {
			undefopt('rut');
		}
	}
}

#**************************************************************************
sub enc {
	my ($self, $val) = @_;
	
	if ($val) {
		$self->opt('enc', $val);
	} else {
		my $enc = $self->opt('enc');
		unless ($enc) {
			$enc = setlocale(LC_CTYPE);
			if ($enc =~ m/\./) {
				$enc =~ s/^[^.]+\.//;
				if ($enc eq '') {
					$enc = 'UTF-8';
				}
			} else {
				$enc = 'UTF-8';
			}
			$self->opt('enc', $enc);
		}
		return $enc;
	}
}

#**************************************************************************
sub mem {
	my ($self, $val) = @_;

	if (defined $val) {
		$self->opt('mem', $val);
	} else {
		my $res = $self->opt('mem');
		if (defined($res)) {
			$res =~ s/[^0-9]+//g;
			unless (($res ne '') && ($res >= 0) && ($res <= 256)) {
				$res = 0;
				$self->opt('mem', $res);
			}
		} else {
			$res = 0;
			$self->opt('mem', $res);
		}
		return $res;
	}
}

#**************************************************************************
sub ovr {
	my ($self, $val) = @_;

	if (defined $val) {
		$self->opt('ovr', ($val) ? 1 : 0);
	} else {
		return ($self->opt('ovr')) ? 1 : 0;
	}
}

#**************************************************************************
sub cas {
	my ($self, $val) = @_;

	if (defined $val) {
		$self->opt('cas', ($val) ? 1 : 0);
	} else {
		return ($self->opt('cas')) ? 1 : 0;
	}
}

#**************************************************************************
sub lin {
	my ($self, $val) = @_;

	if (defined $val) {
		$self->opt('lin', ($val) ? 1 : 0);
	} else {
		return ($self->opt('lin')) ? 1 : 0;
	}
}

#**************************************************************************
sub pfx {
	my ($self, $val) = @_;

	if (defined $val) {
		$self->opt('pfx', $val);
	} else {
		my $res = $self->opt('pfx');
		unless (defined $res) {
			$res = $self->pairs;
			if (defined($res) && (ref($res) eq 'HASH')) {
				my $nod = $self->iln;
				if ((ref($res) eq 'HASH') && exists($res->{$nod}) && ($res->{$nod} ne '')) {
					$res = $res->{$nod};
				} else {
					undefopt('pfx');
				}
				$nod = $self->oln;
				if ((ref($res) eq 'HASH') && exists($res->{$nod}) && ($res->{$nod} ne '')) {
					$res = $res->{$nod};
				} else {
					undefopt('pfx');
				}
				$nod = $self->thm;
				if ((ref($res) eq 'HASH') && exists($res->{$nod}) && ($res->{$nod} ne '')) {
					$res = $res->{$nod};
				} else {
					undefopt('pfx');
				}
			} else {
				undefopt('pfx');
			}
		}
		return $res;
	}
}

#**************************************************************************
sub pairs {
	my ($self, $val) = @_;

	if (defined $val) {
		$self->opt('pairs', $val) if (ref($val) eq 'HASH');
	} else {
		my $res = $self->opt('pairs');
		unless (defined($res) && (ref($res) eq 'HASH')) {
			$res = {};
			$self->opt('pairs', $res);
		}
		return $res;
	}
}

#**************************************************************************
sub pairs2add {
	my ($self, $iln, $oln, $thm, $pfx) = @_;
	$thm ||= 'common';
	
	my $ret = 0;	
	my $par = $self->opt('pairs');
	if (exists($par->{$iln}) && exists($par->{$iln}->{$oln}) && exists($par->{$iln}->{$oln}->{$thm})) {
		if ($par->{$iln}->{$oln}->{$thm} ne $pfx) {
			$par->{$iln}->{$oln}->{$thm} = $pfx;
			$ret = 1;
		}
	} else {
		$par->{$iln}->{$oln}->{$thm} = $pfx;
		$ret = 2;
	}

	return $ret;
}

#**************************************************************************
sub pairs2del {
	my ($self, $iln, $oln, $thm) = @_;
	$thm ||= 'common';
	
	my $ret = 0;	
	my $par = $self->opt('pairs');
	if (exists($par->{$iln}) && exists($par->{$iln}->{$oln}) && exists($par->{$iln}->{$oln}->{$thm})) {
		++$ret;
		delete($par->{$iln}->{$oln}->{$thm});
		if (scalar(keys %{$par->{$iln}->{$oln}}) == 0) {
			delete($par->{$iln}->{$oln});
			if (scalar(keys %{$par->{$iln}}) == 0) {
				delete($par->{$iln});
			}
		}
	}

	return $ret;
}

#**************************************************************************
return 1;
