package Lingua::Pere::Dict::Trans;

use locale;
use strict;

use vars qw(@ISA $VERSION);

use Exporter;
use Lingua::Pere::Dict;

$VERSION = "0.2.0";
@ISA = qw(Exporter Lingua::Pere::Dict);

use POSIX;
use IO::File;
use DB_File;
use Text::CSV_XS;
use Lingua::Pere::Misc qw(
	randval
	str2hex
	hex2str
);
use Lingua::Pere::Misc::File qw(
	makedir
	openfile
);
use Lingua::Pere::Misc::Text qw(
	enctext
	dectext
	text2parse
	parse2text
);
use Lingua::Pere::Misc::Date qw(
	nowdate
);

#**************************************************************************
sub new {
	my ($class, $pfx, $new, $mem) = @_;

	my $mod = O_RDONLY;
	if ($new) {
		$mod = O_CREAT|O_RDWR;
		makedir($pfx) unless (-d $pfx);
	}

	my $self = $class->SUPER::new({ wdir => $pfx, mode => $mod, csiz => ($mem) ? $mem*1024*1024 : undef });

	if ($new) {
		$self->empty;
	}

	return $self;
}

#**************************************************************************
sub empty {
	my ($self) = @_;

	$self->kill([qw(conf tran seqf seqb wdsf wdsb)]);
}

#**************************************************************************
sub src2dic {
	my ($self, $fil, $prg) = @_;
	
	my $fh = openfile($fil, "r");
	if ($fh) {
		$prg = sub { } if (ref($prg) ne 'CODE');

		my $csv = Text::CSV_XS->new({'binary' => 1});

		if ($csv->parse($fh->getline)) {
			my ($sig, $ver, $tim) = $csv->fields;
			if ($sig ne 'PERE_DICT_SOURCE') {
				die("It is not Pere dictionary source file!\n");
			}
			if ($ver ne '0.2.0') {
				die("Incompatibile version of Pere dictionary source file!\n");
			}
		} else {
			my $err = $csv->error_input;
			die("Error in Text::CSV_XS::parse: $err\n");
		}

		my $htran = $self->geth('tran');
		while (<$fh>) {
			chomp;
			my $line = $_;
			if ($csv->parse($line)) {
				my ($wi, @ao) = $csv->fields;

				if (scalar(@ao) > 0) {
					$wi = enctext($wi);
					if (defined($wi) && ($wi ne '') && ($wi =~ /[[:alpha:]]/)) {
						my $ci = $self->encseq($self->encwords(text2parse($wi)));
						foreach my $wo (@ao) {
							$wo = enctext($wo);
							if (defined($wo) && ($wo ne '') && ($wo =~ /[[:alpha:]]/)) {
								my $co = $self->encseq($self->encwords(text2parse($wo)));

								if (exists $htran->{$ci}) {
									$htran->{$ci} .= $co;
								} else {
									$htran->{$ci} = $co;
								}
							}
						}
					}
				}
			} else {
				my $err = $csv->error_input;
				warn("Error in Text::CSV_XS::parse: $err\n");
			}
			&{$prg};
		}

		$fh->close;
	}
}

#**************************************************************************
sub dic2src {
	my ($self, $fil, $prg) = @_;

	my $fh = openfile($fil, "w");
	if ($fh) {
		$prg = sub { } if (ref($prg) ne 'CODE');

		my $csv = Text::CSV_XS->new({'binary' => 1});

		if ($csv->combine('PERE_DICT_SOURCE', '0.2.0', nowdate())) {
			printf($fh "%s\n", $csv->string);
		} else {
			my $err = $csv->error_input;
			die("Error in Text::CSV_XS::combine: $err\n");
		}

		my $dtran = $self->getd('tran');
		my $st = 0;
		my $ci = '';
		my $hz = undef;
		for ($st = $dtran->seq($ci, $hz, R_FIRST); $st == 0 ; $st = $dtran->seq($ci, $hz, R_NEXT)) {
			my $wi = dectext(parse2text($self->decwords($self->decseq($ci))));

			my @ao = unpack('l*', $hz);
			foreach my $uo (@ao) {
				$uo = dectext(parse2text($self->decwords($self->decseq(pack('l',$uo)))));
			}

			if ($csv->combine($wi, @ao)) {
				printf($fh "%s\n", $csv->string);
			} else {
				my $err = $csv->error_input;
				warn("Error in Text::CSV_XS::combine: $err\n");
			}

			&{$prg};
		}
		$fh->close;
	}
}

#**************************************************************************
sub dic2dmp {
	my ($self, $fil, $prg) = @_;

	my $fh = openfile($fil, "w");
	if ($fh) {
		$prg = sub { } if (ref($prg) ne 'CODE');

		my $csv = Text::CSV_XS->new({'binary' => 1});

		if ($csv->combine('PERE_DICT_DUMP', '0.2.0', nowdate())) {
			printf($fh "%s\n", $csv->string);
		} else {
			my $err = $csv->error_input;
			die("Error in Text::CSV_XS::combine: $err\n");
		}

		my @nma = qw(conf seqb seqf tran wdsb wdsf);
		foreach my $nam (@nma) {
			my $dh = $self->getd($nam);

			my $sta = 0;
			my $key = '';
			my $val = undef;
			for ($sta = $dh->seq($key, $val, R_FIRST); $sta == 0 ; $sta = $dh->seq($key, $val, R_NEXT)) {
				if ($csv->combine($nam, str2hex($key), str2hex($val))) {
					printf($fh "%s\n", $csv->string);
				} else {
					my $err = $csv->error_input;
					warn("Error in Text::CSV_XS::combine: $err\n");
				}
				&{$prg};
			}
		}
		$fh->close;
	}
}

#**************************************************************************
sub dmp2dic {
	my ($self, $fil, $prg) = @_;

	my $fh = openfile($fil, "r");
	if ($fh) {
		$prg = sub { } if (ref($prg) ne 'CODE');

		my $csv = Text::CSV_XS->new({'binary' => 1});

		if ($csv->parse($fh->getline)) {
			my ($sig, $ver, $tim) = $csv->fields;
			if ($sig ne 'PERE_DICT_DUMP') {
				die("It is not Pere dictionary dump file!\n");
			}
			if ($ver ne '0.2.0') {
				die("Incompatibile version of Pere dictionary dump file!\n");
			}
		} else {
			my $err = $csv->error_input;
			die("Error in Text::CSV_XS::parse: $err\n");
		}

		my $cnm = '';
		my $dfl = undef;
		while (<$fh>) {
			chomp;
			my $line = $_;
			if ($csv->parse($line)) {
				my ($nam, $key, $val) = $csv->fields;
				
				if ($nam ne $cnm) {
					$dfl = $self->getd($nam);
					$cnm = $nam;
				}

				my $sta = $dfl->put(hex2str($key), hex2str($val));
				if ($sta != 0) {
					warn("Error in DB_File::put: $sta\n");
				}
			} else {
				my $err = $csv->error_input;
				warn("Error in Text::CSV_XS::parse: $err\n");
			}
			&{$prg};
		}
		$fh->close;
	}
}

#*************************************************************
sub translate {
	my ($self, $org, $cas) = @_;

	my $htran = $self->geth('tran');
	my $hseqf = $self->geth('seqf');
	my $hwdsf = $self->geth("wdsf");

	my @funa = (
		sub { return $_[0]; },
		sub { return lc($_[0]); },
		sub { return lc($_[0]); },
		sub { return uc($_[0]); },
		sub { return lcfirst(uc($_[0])); },
		sub { return ucfirst(lc($_[0])); },
	);

	my $tra = [];
	my $i = 0;
	my $j = 0;
	while ($i <= $#{$org}) {
		my $tr = 0;
		my $co = '';
		my $ci = '';
		my $func = '';
		my $ud = $org->[$i++];
		my $wd = dectext($ud);

		if (exists $hwdsf->{$wd}) {
			$ci = $hwdsf->{$wd};
			if (exists $htran->{$ci}) {
				$co = randval($htran->{$ci});
				++$tr;
				$j = $i;

				while ($i <= $#{$org}) {
					$wd = dectext($org->[$i++]);
					if (exists $hwdsf->{$wd}) {
						my $cb = $ci.$hwdsf->{$wd};
						if (exists $hseqf->{$cb}) {
							$cb = $hseqf->{$cb};
							if (exists $htran->{$cb}) {
								$ci = $cb;
								$co = randval($htran->{$ci});
								++$tr;
								$j = $i;
#-----------------------------------
#							} else {
#								last;
#-----------------------------------
							}
						} else {
							last;
						}
					} else {
						last;
					}
				}
			}
#-----------------------------------
		} elsif (!$cas) {
			my $ca = 0;
			while ((++$ca <= $#funa) && ($funa[$ca]->($ud) ne $ud)) {};

			if ($ca <= $#funa) {
				$func = $funa[$ca];

				my $cr = '';
				my $k = 1;
				while (($cr eq '') && ($k <= $#funa)) {
					if ($k != $ca) {
						$wd = dectext($funa[$k]->($ud));
						if (exists $hwdsf->{$wd}) {
							$cr = $hwdsf->{$wd};
						}
					}
					++$k;
				}

				if (($cr ne '') && (exists $htran->{$cr})) {
					$co = randval($htran->{$cr});
					++$tr;
					$j = $i;
				}
			}
#-----------------------------------
		}
		
		if ($co ne '') {
			my $so = $self->decwords($self->decseq($co));
			if ($func ne '') {
				foreach my $ww (@{$so}) {
					$ww = $func->($ww);
				}
			}

			push(@{$tra}, [$tr, $so]);

			$i = $j;
		} else {
			push(@{$tra}, [$tr, [$org->[$i-1]]]);
		}
	}

	return $tra;
}

#**************************************************************************
return 1;
__END__
