package Lingua::Pere::Util;

use locale;
use strict;

use vars qw(@ISA $VERSION @EXPORT @EXPORT_OK);
use vars qw(%FUNCS $STXU $ETXU $STXA $ETXA $SHTM $EHTM $SOOO $EOOO @CON);

use Exporter;

$VERSION = "0.2.0";
@ISA = qw(Exporter);
@EXPORT = ();
@EXPORT_OK = qw(
	funcs4trans
	trans
	trans4text
	trans4html
	trans4ooo
	trans4rtf
	trans4msw2003
	trans4mse2003
	trans4docbook
);

use POSIX;
use HTML::SimpleParse;
use IO::Scalar;
use Archive::Zip qw( 
	:ERROR_CODES 
	:CONSTANTS 
);
use Lingua::Pere::Dict::Trans;
use Lingua::Pere::Util::XML;
use Lingua::Pere::Util::RTF;
use Lingua::Pere::Misc::File qw(
	openfile
	readfile
	writefile
);
use Lingua::Pere::Misc::Text qw(
	enctext
	dectext
	codtext
	preptext
	text2parse
	parse2text
);
use Lingua::Pere::Misc::HTML qw(
	decentis
	encentis
);
use Lingua::Pere::Misc::MIME qw(
	magic2mime
	mime2descr
);

$STXU = "\x{02}";
$ETXU = "\x{03}";

$STXA = "\x02";
$ETXA = "\x03";

$SHTM = '<FONT COLOR="red">';
$EHTM = '</FONT>';

$SOOO = '<text:span text:style-name="PERE4UNKWORD">';
$EOOO = '</text:span>';

@CON = ('content.xml', 'styles.xml');

#**************************************************************************
sub trans4text {
	my ($ifl, $ofl, $prg, $pfx, $mem, $cas, $enc, $lin) = @_;
	$prg ||= sub {};

	my $ihn = openfile($ifl, 'r');
	if (defined($ihn)) {
		my $ohn = openfile($ofl, 'w');
		if (defined($ohn)) {
			my $org = readfile($ihn);
			my $dic = Lingua::Pere::Dict::Trans->new($pfx, 0, $mem);

			$org =~ s/^(?:\r?\n)+//;
			$org =~ s/(?:\r?\n)+$//;
			my @txa;
			if ($lin) {
				@txa = split(/(?:\s*\r?\n)+/, $org);
			} else {
				@txa = split(/\s*\r?\n(?:\s*\r?\n)+/, $org);
			}

			my $res = [];
			foreach my $row (@txa) {
				my ($ltx, $txt, $rtx) = preptext(enctext($row, $enc));
				if (($txt ne '') && ($txt =~ /[[:alpha:]]/)) {
					my $tra = [];
					foreach my $rwt (@{$dic->translate(text2parse($txt), $cas)}) {
						push(@{$tra}, @{$rwt->[1]});
					}
					push(@{$res}, dectext($ltx.parse2text($tra).$rtx, $enc));
				} else {
					push(@{$res}, $row);
				}
				&{$prg};
			}

			if ($lin) {
				$res = join("\n", @{$res})."\n";
			} else {
				$res = join("\n\n", @{$res})."\n";
			}

			writefile($ohn, $res);
		}
	}
}

#**************************************************************************
sub trans4html {
	my ($ifl, $ofl, $prg, $pfx, $mem, $cas, $enc) = @_;
	$prg ||= sub {};

	my $ihn = openfile($ifl, 'r');
	if (defined($ihn)) {
		my $ohn = openfile($ofl, 'w');
		if (defined($ohn)) {
			my $org = readfile($ihn);
			if ($org =~ m/<META\s+[^>]*?CONTENT=(["'])text\/html; charset=(.+?)\1[^>]*?>/is) {
				$enc = $2;
				$org =~ s/<META\s+[^>]*?CONTENT=(["'])text\/html; charset=(.+?)\1[^>]*?>/<META HTTP-EQUIV="CONTENT-TYPE" CONTENT="text\/html; charset=utf-8">/is
			} else {
				$org =~ s/(<HEAD>)/$1\n<META HTTP-EQUIV="CONTENT-TYPE" CONTENT="text\/html; charset=utf-8">/is;
			}

			if ($org =~ m/<style\s+type="text\/css">.*?<\/style>/is) {
				$org =~ s/(<\/style>)/span.pere4unkword {color: red}\n$1/i;
				$SHTM = '<SPAN class="pere4unkword">';
				$EHTM = '</SPAN>';
			}

			my $dic = Lingua::Pere::Dict::Trans->new($pfx, 0, $mem);
	
			my $htm = HTML::SimpleParse->new($org, 'fix_case' => 0);
			my $pre = 0;
			my $res = [];
			foreach my $nod ($htm->tree) {
				my $typ = $nod->{type};
				if ($typ eq 'starttag') {
					$pre++ if ($nod->{content} =~ /^\s*(PRE|HEAD|CODE|NOPERE)/is);
				}
				if ($typ eq 'endtag') {
					$pre-- if ($nod->{content} =~ /^\s*\/(PRE|HEAD|CODE|NOPERE)/is);
				}

				if ($typ eq 'text') {
					if ($pre < 1) {
						my $row = enctext(decentis(codtext($nod->{content}, $enc)));
						if (defined($row) && ($row ne '') && ($row =~ /[[:alpha:]]/)) {
							my ($ltx, $txt, $rtx) = preptext($row);
							my $tra = [];
							foreach my $rwt (@{$dic->translate(text2parse($txt), $cas)}) {
								if ($rwt->[0] < 1) {
									foreach my $rw (@{$rwt->[1]}) {
										if ($rw =~ m/[[:alpha:]]/) {
											$rw = $STXU.$rw;
											$rw .= $ETXU;
										}
									}
								}
								push(@{$tra}, @{$rwt->[1]});
							}
							$row = encentis(dectext($ltx.parse2text($tra).$rtx));
							$row =~ s/[$STXA]+/$SHTM/g;
							$row =~ s/[$ETXA]+/$EHTM/g;
							$nod->{content} = $row;
						}
					} else {
						$nod->{content} = codtext($nod->{content}, $enc);
					}
				}
				&{$prg};
			}
			writefile($ohn, $htm->get_output);
		}
	}
}
	
#**************************************************************************
my $dat4trans4mse2003 = 0;
sub _huk4trans4mse2003 {
	my ($self, $sta, $elm, $att) = @_;
	
	if ($sta) {
		if ($elm =~ m/^(Data|ss:Data)$/) {
			if (!exists($att->{'ss:Type'}) || ($att->{'ss:Type'} eq 'String')) {
				return 1;
			} else {
				++$dat4trans4mse2003;
			}
		}
	} elsif ($elm =~ m/^(Data|ss:Data)$/) {
		if ($dat4trans4mse2003 > 0) {
			--$dat4trans4mse2003;
		} else {
			return -1
		} 
	}

	return 0;
}

#**************************************************************************
sub trans4mse2003 {
	my ($ifl, $ofl, $prg, $pfx, $mem, $cas, $enc) = @_;
	$prg ||= sub {};

	my $dic = Lingua::Pere::Dict::Trans->new($pfx, 0, $mem);
	my $xml = Lingua::Pere::Util::XML->new(\&_huk4trans4mse2003, $dic, $cas, $enc);
	$xml->parse($ifl, $ofl, $prg);
}

#**************************************************************************
sub _huk4trans4msw2003 {
	my ($self, $sta, $elm, $att) = @_;
	
	if ($sta) {
		if ($elm =~ m/^(w:t|o:Title)$/) {
			return 1;
		}
	} elsif ($elm =~ m/^(w:t|o:Title)$/) {
		return -1
	}

	return 0;
}

#**************************************************************************
sub trans4msw2003 {
	my ($ifl, $ofl, $prg, $pfx, $mem, $cas, $enc) = @_;
	$prg ||= sub {};

	my $dic = Lingua::Pere::Dict::Trans->new($pfx, 0, $mem);
	my $xml = Lingua::Pere::Util::XML->new(\&_huk4trans4msw2003, $dic, $cas, $enc);
	$xml->parse($ifl, $ofl, $prg);
}

#**************************************************************************
sub _huk4trans4docbook {
	my ($self, $sta, $elm, $att) = @_;
	
	if ($sta) {
		if ($elm =~ m/^(para|title|entry)$/) {
			return 1;
		} elsif ($elm =~ m/^(acronym|alt|uri|programlisting|code|example|tag|application|filename|menuchoice|guimenu|guisubmenu|guimenuitem|guibutton|screen|command|option|citation|literal)$/) {
			return -1
		}
	} elsif ($elm =~ m/^(para|title|entry)$/) {
		return -1
	} elsif ($elm =~ m/^(acronym|alt|uri|programlisting|code|example|tag|application|filename|menuchoice|guimenu|guisubmenu|guimenuitem|guibutton|screen|command|option|citation|literal)$/) {
			return 1
	}

	return 0;
}

#**************************************************************************
sub trans4docbook {
	my ($ifl, $ofl, $prg, $pfx, $mem, $cas, $enc) = @_;
	$prg ||= sub {};

	my $dic = Lingua::Pere::Dict::Trans->new($pfx, 0, $mem);
	my $xml = Lingua::Pere::Util::XML->new(\&_huk4trans4docbook, $dic, $cas, $enc);
	$xml->parse($ifl, $ofl, $prg);
}

#**************************************************************************
sub _dopara4trans4ooo {
	my ($dic, $prg, $org, $cas) = @_;
	
	my @par = split(m/(<[^>]+>)/, $org);
	foreach my $chu (@par) {
		unless ($chu =~ m/^<.*>$/) {
			my $row = enctext(decentis($chu));
			if (defined($row) && ($row ne '')) {
#			if (defined($row) && ($row ne '') && ($row =~ m/[[:alnum:]]/)) {
				my ($ltx, $txt, $rtx) = preptext($row);
				my $tra = [];
				foreach my $rwt (@{$dic->translate(text2parse($txt), $cas)}) {
#---------------------------------------------
					if ($rwt->[0] < 1) {
						foreach my $rw (@{$rwt->[1]}) {
							if ($rw =~ m/[[:alpha:]]/) {
								$rw = $STXU.$rw;
								$rw .= $ETXU;
							}
						}
					}
#---------------------------------------------
					push(@{$tra}, @{$rwt->[1]});
				}
				$chu = encentis(dectext($ltx.parse2text($tra).$rtx));
#---------------------------------------------
				$chu =~ s/[$STXA]+/$SOOO/g;
				$chu =~ s/[$ETXA]+/$EOOO/g;
#---------------------------------------------
			}
		}
		&{$prg};
	}
	
	return join('', @par);
}

#**************************************************************************
sub _dofile4trans4ooo {
	my ($zip, $prg, $con, $dic, $cas, $enc) = @_;

	my $txt = $zip->contents($con);
	if (defined($txt) && ($txt ne '')) {

#---------------------------------------------
		unless ($txt =~ m/<office:automatic-styles>/i) {
			$txt =~ s/(<office:body>)/<office:automatic-styles><\/office:automatic-styles>$1/i
		}
		$txt =~ s/(<\/office:automatic-styles>)/<style:style style:name="PERE4UNKWORD" style:family="text"><style:text-properties fo:color="#FF0000"\/><\/style:style>$1/i;
#---------------------------------------------

		$txt =~ s/(<(text:p|text:h)(?:\s+[^>]+)*>)(.*?)(<\/\2>)/$1._dopara4trans4ooo($dic, $prg, $3, $cas).$4/ge;

		$zip->contents($con, $txt);
		
		return 1;
	} else {
		warn "Can not extract '$con'\n";
	}

	return 0;
}

#**************************************************************************
sub trans4ooo {
	my ($ifl, $ofl, $prg, $pfx, $mem, $cas, $enc) = @_;
	$prg ||= sub {};

	my $ihn = openfile($ifl, 'r');
	if (defined($ihn)) {
		my $ohn = openfile($ofl, 'w');
		if (defined($ohn)) {

			my $zip = Archive::Zip->new();
			if ($zip->readFromFileHandle($ihn) == AZ_OK) {
				my $dic = Lingua::Pere::Dict::Trans->new($pfx, 0, $mem);
				my $cnt = 0;
				foreach my $con (@CON) {
					last if (_dofile4trans4ooo($zip, $prg, $con, $dic, $cas, $enc) < 1);
					++$cnt;
				}

				if ($cnt == scalar(@CON)) {
					unless ($zip->writeToFileHandle($ohn) == AZ_OK) {
						carp("Can not create new document\n");
					}
				}
			} else {
				carp("Can not open document: $!");
			}

		}
	}
}

#**************************************************************************
sub _dopara4trans4rtf {
	my ($row, $dic, $cas) = @_;

	if ($row =~ m/[[:alnum:]]/) {
		my ($ltx, $txt, $rtx) = preptext($row);
		my $tra = [];
		foreach my $rwt (@{$dic->translate(text2parse($txt), $cas)}) {
			push(@{$tra}, @{$rwt->[1]});
		}
		$row = $ltx.parse2text($tra).$rtx;
	}

	return $row;
}

#**************************************************************************
sub trans4rtf {
	my ($ifl, $ofl, $prg, $pfx, $mem, $cas, $enc) = @_;
	$prg ||= sub {};

	my $ihn = openfile($ifl, 'r');
	if (defined($ihn)) {
		my $ohn = openfile($ofl, 'w');
		if (defined($ohn)) {
			my $org = readfile($ihn);

			my $res = '';
			if (defined($org) && ($org ne '')) {
				my $dic = Lingua::Pere::Dict::Trans->new($pfx, 0, $mem);
				my $rtf = Lingua::Pere::Util::RTF->new($prg, sub { return _dopara4trans4rtf($_[0], $dic, $cas) }, $enc);
				$res = $rtf->execute($org);
			}

			writefile($ohn, $res);
		}
	}
}

#**************************************************************************
%FUNCS  = (
	'application/vnd.oasis.opendocument.text'                  => \&trans4ooo,
	'application/vnd.oasis.opendocument.text-template'         => \&trans4ooo,
	'application/vnd.oasis.opendocument.text-web'              => \&trans4ooo,
	'application/vnd.oasis.opendocument.text-master'           => \&trans4ooo,
	'application/vnd.oasis.opendocument.graphics'              => \&trans4ooo,
	'application/vnd.oasis.opendocument.graphics-template'     => \&trans4ooo,
	'application/vnd.oasis.opendocument.presentation'          => \&trans4ooo,
	'application/vnd.oasis.opendocument.presentation-template' => \&trans4ooo,
	'application/vnd.oasis.opendocument.spreadsheet'           => \&trans4ooo,
	'application/vnd.oasis.opendocument.spreadsheet-template'  => \&trans4ooo,
#	'application/vnd.oasis.opendocument.chart'                 => undef,
#	'application/vnd.oasis.opendocument.formula'               => undef,
#	'application/vnd.oasis.opendocument.image'                 => undef,
	'application/vnd.sun.xml.writer'                           => \&trans4ooo,
	'application/vnd.sun.xml.writer.template'                  => \&trans4ooo,
	'application/vnd.sun.xml.writer.global'                    => \&trans4ooo,
	'application/vnd.sun.xml.calc'                             => \&trans4ooo,
	'application/vnd.sun.xml.calc.template'                    => \&trans4ooo,
	'application/vnd.sun.xml.impress'                          => \&trans4ooo,
	'application/vnd.sun.xml.impress.template'                 => \&trans4ooo,
	'application/vnd.sun.xml.draw'                             => \&trans4ooo,
	'application/vnd.sun.xml.draw.template'                    => \&trans4ooo,
#	'application/vnd.sun.xml.math'                             => undef,
#	'application/vnd.sun.xml.base'                             => undef,
	'application/rtf'                                          => \&trans4rtf,
	'application/xhtml+xml'                                    => \&trans4html,
	'text/html'                                                => \&trans4html,
	'text/plain'                                               => \&trans4text,
	'application/vnd.ms-word.xml'                              => \&trans4msw2003,
	'application/vnd.ms-excel.xml'                             => \&trans4mse2003,
	'application/vnd.docbook.xml'                              => \&trans4docbook,
);

#**************************************************************************
sub funcs4trans {
	return \%FUNCS;
}

#**************************************************************************
sub trans {
	my ($ifl, $ofl, $prg, $pfx, $mem, $cas, $enc, $lin) = @_;
	$prg ||= sub {};

	my $ihn = openfile($ifl, 'r');
	if (defined($ihn)) {
		my $dsc = '';
		my $mag = '';
		$ihn->read($mag, 1024);
		$ihn->seek(0, 0);
		my $mim = magic2mime($mag);
		if ($mim) {
			$dsc = mime2descr($mim);
		} else {
			$mim = 'text/plain';
			$dsc = "Unknown content type. Use default: ".mime2descr($mim);
		}
		print "$dsc\n";

		if (exists $FUNCS{$mim}) {
			$FUNCS{$mim}->($ihn, $ofl, $prg, $pfx, $mem, $cas, $enc, $lin);
		} else {
			print "Unsupported content type\n";
		}
	}
}

#**************************************************************************
return 1;
